/*
Funkci: 			USB HID Custom pldaprogram
PIC:	 			18F2550
Bootloader:			Nincs
USB max. sebessge:	64KBjt/sec (Egy csomag mrete: 64Bjt)
LED1:				LATBbits.LATB5 (26. lb) (1Kohm ellenllssal sorbaktve a LED. Katd testre)
PC program:			LED1-et kapcsolja be-ki, kiolvassa az azonostkat, fogadja a kldtt teszt csomagokat.

*/

/** INCLUDES *******************************************************/
#include "Compiler.h"
#include "PIC18F2550_Config.h"
#include "GenericTypeDefs.h"
#include "usb_config.h"
#include "usb_device.h"
#include "usb.h"
#include "HardwareProfile.h"
#include "usb_function_hid.h"
#include "Global_Constans.h"
#include "SKType.h"				//Tpus deklarlsok    
    
/** Vltozk ******************************************************/
#pragma udata access ACCESS_section
	volatile near unsigned int USB_P_Counter;
	USB_HANDLE USBOutHandle = 0;
	USB_HANDLE USBInHandle = 0;

#pragma udata BUF512 //lsd: 18f2550sk.lkr
	unsigned char ReceivedDataBuffer[64];
	unsigned char ToSendDataBuffer[64];

//Tpusok pldnyostsa
	TFLAGbits_1 FLAGbits_1;	//TFLAGbits_1 Pldnyostsa
#define FLAG_1 FLAGbits_1.x

/** Prototpusok deklarlsa *********************************************/

//Interrupts fggvnyek Prototpusai
	void low_int(void);
	void high_int(void);

//Port, perifria szubrutinok prototpusai
	void Port_cfg(void);  		//Port_cfg.c
	void Pheriph_cfg(void);		//Pheriph_cfg.c
	void INT_cfg(void);			//Interrupt_cfg.c
	void Variable_res(void);	//Vltozk alaphelyzetbe lltsa

//** INTERRUPT Pontok rgztse ******************
/***********************************************************/
//Magas prioritsi megszakts pont
#pragma code high_vector=0x08  //Assemly kd elhelyezse a megszakts cmen
	void interrupt_at_high_vector(void)
	{
	_asm GOTO high_int _endasm
	}//END SUB
	
// Alacsony prioritsi megszakts pont
#pragma code low_vector=0x18  //Assemly kd elhelyezse a megszakts cmen
	void interrupt_at_low_vector(void)
	{
	_asm GOTO low_int _endasm
	}//END SUB
	
/* Visszatrs az alaprtelmezett cm kiosztshoz */
#pragma code 
/***********************************************************/

//________________________________________________________
//			            	main						  |	
//________________________________________________________|
void main(void)
{
	Port_cfg();			// Portok belltsa
	Pheriph_cfg();		// Perifrik belltsa

    USBDeviceInit();	// usb_device.c.  Initializes USB module SFRs and firmware variables to known states.

	INT_cfg();			// Interruptok belltsa

    Variable_res();		// Vltozk alaphelyzetbe lltsa

//________________________________________________________________________________
//**************** Fciklus ******************************************************
    while(1)
    {
    // USB llapotvizsglata
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1))
	{}
	else
	{
	    if(!HIDRxHandleBusy(USBOutHandle))
	    {   
			// LED parancs vtele s vgrehajtsa
	        switch(ReceivedDataBuffer[0])
	        {
	            case 0x80:  				// Toggle LEDs
	                mLED_1_Toggle();
					FLAGbits_1.LED_Toggle_Flg = Set;
			    break;
			}
	        //Re-arm the OUT endpoint for the next packet
	        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
	    }        
			// LED visszajelzs elkldse PC-nek.
			if (FLAGbits_1.LED_Toggle_Flg)
			{
				if(!HIDTxHandleBusy(USBInHandle))
				{
					if (mLED_1)
					{
						ToSendDataBuffer[1] = 113; 	//Ha a LED vilgt
					}
					else
					{
						ToSendDataBuffer[1] = 114;
					}
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer,64);
					FLAGbits_1.LED_Toggle_Flg = Clear;
		        }
			}
			// 100Hz temben csomagot kldnk a PC programnak.
			if	(FLAGbits_1.msDely)	// Megszaktsban lltdik be. 	
			{	
				ToSendDataBuffer[0] = 112;		// zenetazonost kd. Konnektlst jelzt villogtatja a PC Form-on.
				
		        if(!HIDTxHandleBusy(USBInHandle))
		        {
		            USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer,64);
					FLAGbits_1.msDely = Clear;	// temjelz trlve
		        }
		  	}
	} //End if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1))
  }//end while
//________________________________________________________________________________
//**************** Fciklus Vge *************************************************
}//end main


//________________________________________________________
//			  Vltozk alaphelyzetbe lltsa			  |	
//________________________________________________________|
void Variable_res(void)
{
	// USB	
    USBOutHandle = 0;
    USBInHandle = 0;
	
	// SK
	USB_P_Counter = 0;
	FLAG_1 = 0;

	ToSendDataBuffer[0]=0;
	ToSendDataBuffer[1]=114;
} // End Variable_res


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and 
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific 
 *					firmware, such as that contained in usb_function_hid.c.
 *
 * Note:            None
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This 
 *					callback function should initialize the endpoints 
 *					for the device's usage according to the current 
 *					configuration.
 *
 * Note:            None
 *******************************************************************/
void USBCBInitEP(void)
{
    //enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    //Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB specifications allow some types of USB
 * 					peripheral devices to wake up a host PC (such
 *					as if it is in a low power suspend to RAM state).
 *					This can be a very useful feature in some
 *					USB applications, such as an Infrared remote
 *					control	receiver.  If a user presses the "power"
 *					button on a remote control, it is nice that the
 *					IR receiver can detect this signalling, and then
 *					send a USB "command" to the PC to wake up.
 *					
 *					The USBCBSendResume() "callback" function is used
 *					to send this special USB signalling which wakes 
 *					up the PC.  This function may be called by
 *					application firmware to wake up the PC.  This
 *					function should only be called when:
 *					
 *					1.  The USB driver used on the host PC supports
 *						the remote wakeup capability.
 *					2.  The USB configuration descriptor indicates
 *						the device is remote wakeup capable in the
 *						bmAttributes field.
 *					3.  The USB host PC is currently sleeping,
 *						and has previously sent your device a SET 
 *						FEATURE setup packet which "armed" the
 *						remote wakeup capability.   
 *
 *					This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            Interrupt vs. Polling
 *                  -Primary clock
 *                  -Secondary clock ***** MAKE NOTES ABOUT THIS *******
 *                   > Can switch to primary first by calling USBCBWakeFromSuspend()
 
 *                  The modifiable section in this routine should be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of 1-13 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at lest 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do
    {
        delay_count--;
    }while(delay_count);
    USBResumeControl = 0;
}

/*********************************************************************/
//Megszaktsok lekezelse

//________________________________________________________
//					  HighInt							  |
//________________________________________________________|
#pragma interrupt high_int
	
void high_int (void)	//Megszakts lekezelse
{
	//USB <-> Timer0
	if(INTCONbits.TMR0IF && INTCONbits.TMR0IE)
	{
		// 6000 Timer0 INT/sec belltsa
		_asm
				NOP
				NOP
				NOP
		//		NOP
		//		NOP
		_endasm
		TMR0L=16;  // TMR0 feltltse kezd rtkkel
		
		// USB polling lekezelse.
		USBDeviceTasks();

		// USB csomag temez	
			USB_P_Counter++;
			if (USB_P_Counter==60)
			{
				USB_P_Counter = Clear;
				FLAGbits_1.msDely = Set; // 100Hz csomagsebessg
			}
	INTCONbits.TMR0IF=Clear; 
	} // End if (INTCONbits.TMR0IF && INTCONbits.TMR0IE)
} // End high_int


//________________________________________________________
//   				  LowInt							  |
//________________________________________________________|
#pragma interruptlow low_int
		
void low_int (void)   //Low megszakts lekezelse
{

} // End low_int


/** EOF main.c *************************************************/

